/**
	Javaフォトレタッチビットマップパネルコンポーネント

	2008/ 1/ 1 宍戸　輝光
*/
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;

public class BMPanel extends JPanel implements MouseMotionListener {

	private JIntFrameBuffer _frame;

	private BufferedImage _img;

	private int _width, _height, _imgWidth, _imgHeight;

	private j8ph _owner;

	private double _vsSize;

	public BMPanel(j8ph arg) {

		_owner = arg;
		_frame = new JIntFrameBuffer(128, 128);

		_vsSize = 1.0;

		addMouseMotionListener(this);

	}

	public boolean setViewSize(double vs) {

		if (vs < 0.1) {
			return false;
		}

		_vsSize = vs;

		_width = (int)(_imgWidth * _vsSize);
		_height = (int)(_imgHeight * _vsSize);

		setSize(new Dimension(_width, _height));

		repaint();

		return true;

	}

	public void setFrameBuffer(JIntFrameBuffer frm) {

		_frame.assign(frm);

		if (_frame.getWidth() != _imgWidth || _frame.getHeight() != _imgHeight) {
			setImageSize(_frame.getWidth(), _frame.getHeight());
		}

		updateImage();
		repaint();

	}

	private void setImageSize(int width,int height) {

		_imgWidth = width;
		_imgHeight = height;

		// BufferedImage再確保
		_img = new BufferedImage(_imgWidth, _imgHeight, BufferedImage.TYPE_INT_RGB);

		_width = (int)(_imgWidth * _vsSize);
		_height = (int)(_imgHeight * _vsSize);

		setSize(new Dimension(_width, _height));

	}

	void updateImage() {

		// BufferedImageのピクセル列設定
		_img.getRaster().setDataElements(0, 0, _imgWidth, _imgHeight, _frame.getBuffer());

	}

	public void paintComponent(Graphics g) {

		if (_vsSize == 1) {
			g.drawImage(_img, 0, 0, this);
		} else {
			g.drawImage(_img, 0, 0, _width, _height, this);
		}

	}

	public Dimension getPreferredSize() {

		return new Dimension(_width, _height);

	}

	public void mouseDragged(MouseEvent e) {}

	// マウスポインタ移動
	public void mouseMoved(MouseEvent e) {

		int px = (int)(e.getX() * (1.0 / _vsSize));
		int py = (int)(e.getY() * (1.0 / _vsSize));

		if (px >= _imgWidth || py >= _imgHeight) {
			return;
		}

		// マウス座標のRGB取得
		int r = _frame.getR(px, py);
		int g = _frame.getG(px, py);
		int b = _frame.getB(px, py);

		// マウス座標のRGB情報を文字列化
		String mes=" (" + px + "," + py + ")" + ":";
		mes = mes + r + "," + g + "," + b;

		// 文字列化したRGB情報をアプリケーションに表示
		_owner.setStatus(mes);

	}

}
