//  32Bitフレームバッファクラス
//
//   2002/12/ 9 宍戸　輝光
import java.awt.*;
import java.awt.image.*;
import java.io.*;
import javax.imageio.*;

class JIntFrameBuffer {

	private int _width,_height;
	private int [] m_aBuffer;
	private int m_iMaxWidth,m_iMaxHeight;

	public JIntFrameBuffer() {

		// 最大サイズ設定
		m_iMaxWidth = 4096;
		m_iMaxHeight = 4096;

		// 128*128で初期化
		reset(128, 128);

	}

	public JIntFrameBuffer(JIntFrameBuffer src) {

		// 最大サイズ設定
		m_iMaxWidth = 4096;
		m_iMaxHeight = 4096;

		if (src == null) {

			reset(128, 128);

			return;

		}

		assign(src.getPixels());

	}

	public JIntFrameBuffer(int iWidth, int iHeight) {

		m_iMaxWidth = 4096;
		m_iMaxHeight = 4096;

		reset(iWidth, iHeight);

	}

	public JIntFrameBuffer(int[] pixels) {

		// 最大サイズ設定
		m_iMaxWidth = 4096;
		m_iMaxHeight = 4096;

		assign(pixels);

	}

	// 指定サイズで初期化
	public boolean reset(int width, int height) {

		if (width < 0 || width > m_iMaxWidth || height < 0 || height > m_iMaxHeight) {
			return false;
		}

		// ピクセル用バッファ再確保
		int[] buf = new int[width * height];

		// 確保に失敗したら戻る
		if (buf == null) {
			return false;
		}

		// ピクセル用バッファ設定
		m_aBuffer = buf;

		// 画像サイズ設定
		_width = width;
		_height = height;

		return true;

	}

	public boolean assign(Image img) {

		int[] data = j8phImageCodec.getPixels(img);

		return assign(data);

	}

	public boolean assign(int width, int height, int[] pixels, int offset) {

		if (width < 1 || width > m_iMaxWidth || height < 1 || height > m_iMaxHeight || pixels == null || pixels.length < width * height + offset) {
			return false;
		}

		if (!reset(width, height)) {
			return false;
		}

		for (int i = 0;i < height;i++) {
			for (int j = 0;j < width;j++) {
				setPixel(j, i, pixels[offset + j + i * width]);
			}
		}

		return true;

	}

	public boolean assign(int[] data) {

		if (data == null || data.length < 3) {
			return false;
		}

		int width = data[0];
		int height = data[1];

		return assign(width, height, data, 2);

	}

	public boolean assign(JIntFrameBuffer arg) {

		if (arg == null || arg.getWidth() < 1 || arg.getHeight() < 1) {
			return false;
		}

		if (!reset(arg.getWidth(), arg.getHeight())) {
			return false;
		}

		System.arraycopy(arg.getBuffer(), 0, m_aBuffer, 0, _width * _height);

		return true;

	}

	public int[] getPixels() {

		if (_width < 1 || _height < 1) {
			return null;
		}

		int[] data = new int[2 + _width * _height];

		if (data == null) {
			return null;
		}

		data[0] = _width;
		data[1] = _height;

		for (int i = 0;i < _height;i++) {
			for (int j = 0;j < _width;j++) {
				data[2 + j + i * _width] = getPixel(j, i) & 0xffffff;
			}
		}

		return data;

	}

	// 横幅取得
	public int getWidth() { return _width; }

	// 高さ取得
	public int getHeight() { return _height; }

	// ピクセル列バッファ取得
	public int[] getBuffer() { return m_aBuffer; }

	public int getColors() {

		if (_width < 1 || _height < 1) {
			return 0;
		}

		byte[] map = new byte[(256 * 256 * 256) / 8];

		if (map == null) {
			return 0;
		}

		int colors = 0;

		for (int i = 0;i < _height;i++) {
			for (int j = 0;j < _width;j++) {

				int rgb = getPixel(j, i) & 0x00ffffff;

				int index = rgb / 8;
				int bit = 1 << (rgb % 8);

				if ((map[index] & bit) == 0) {

					map[index] |= bit;

					colors++;

				}

			}
		}

		return colors;

	}

	// (iX,iY)の値を取得
	public int getPixel(int iX,int iY) {

		if (iX<0 || iX>=getWidth() || iY<0 || iY>=getHeight())
			return 0;

		return m_aBuffer[iX+iY*_width];

	}

	// (IX,iY)のR成分取得
	public int getR(int iX,int iY) {

		if (iX<0 || iX>=getWidth() || iY<0 || iY>=getHeight())
			return 0;

		return (getPixel(iX,iY) & 0x00ff0000) >> 16;

	}

	// (IX,iY)のG成分取得
	public int getG(int iX,int iY) {

		if (iX<0 || iX>=getWidth() || iY<0 || iY>=getHeight())
			return 0;

		return (getPixel(iX,iY) & 0x0000ff00) >> 8;

	}

	// (IX,iY)のB成分取得
	public int getB(int iX,int iY) {

		if (iX<0 || iX>=getWidth() || iY<0 || iY>=getHeight())
			return 0;

		return getPixel(iX,iY) & 0x000000ff;

	}

	// (iX,iY)のピクセル設定
	public boolean setPixel(int iX,int iY,int cl) {

		if (iX < 0 || iX >= getWidth() || iY < 0 || iY >= getHeight()) {
			return false;
		}

		m_aBuffer[iX+iY * _width] = 0x00ffffff & cl;

		return true;

	}

	// (iX,iY)のR成分設定
	public boolean setR(int iX,int iY,int iR) {

		if (iX<0 || iX>=_width || iY<0 || iY>=_height)
			return false;

		if (iR<0)
			iR=0;
		if (iR>255)
			iR=255;

		iR=iR << 16;

		int iCl=getPixel(iX,iY);

		iCl=iCl & 0x0000ffff;
		iCl=iCl | iR;

		setPixel(iX,iY,iCl);

		return true;

	}

	// (iX,iY)のG成分設定
	public boolean setG(int iX,int iY,int iG) {

		if (iX<0 || iX>=_width || iY<0 || iY>=_height)
			return false;

		if (iG<0)
			iG=0;
		if (iG>255)
			iG=255;

		iG=iG << 8;

		int iCl=getPixel(iX,iY);

		iCl=iCl & 0x00ff00ff;
		iCl=iCl | iG;

		setPixel(iX,iY,iCl);

		return true;

	}

	// (iX,iY)のB成分設定
	public boolean setB(int iX,int iY,int iB) {

		if (iX<0 || iX>=_width || iY<0 || iY>=_height) {
			return false;
		}

		if (iB<0)
			iB=0;
		if (iB>255)
			iB=255;

		int iCl=getPixel(iX,iY);

		iCl=iCl & 0x00ffff00;
		iCl=iCl | iB;

		setPixel(iX,iY,iCl);

		return true;

	}

	public boolean addRGB(int x, int y, int dr, int dg, int db) {

		if (x < 0 || y >= _width || y < 0 || y >= _height) {
			return false;
		}

		setR(x, y, getR(x, y) + dr);
		setG(x, y, getG(x, y) + dg);
		setB(x, y, getB(x, y) + db);

		return true;

	}

	public boolean addHSB(int x, int y, int dh, int ds, int db) {

		if (x < 0 || y >= _width || y < 0 || y >= _height) {
			return false;
		}

		float[] hsb = Color.RGBtoHSB(getR(x, y), getG(x, y), getB(x, y), null);

		if (hsb == null || hsb.length < 3) {
			return false;
		}

		if (hsb[0] < 0) {
			hsb[0] = -hsb[0];
		}

		hsb[0] -= Math.floor(hsb[0]);

		float h = (int)(hsb[0] * 256) + dh;
		float s = (int)(hsb[1] * 256) + ds;
		float b = (int)(hsb[2] * 256) + db;

		if (s < 0) {
			s = 0;
		}

		if (s > 255) {
			s = 255;
		}

		if (b < 0) {
			b = 0;
		}

		if (b > 255) {
			b = 255;
		}

		h /= 256.0;
		s /= 256.0;
		b /= 256.0;

		setPixel(x, y, Color.HSBtoRGB(h, s, b));

		return true;

	}

	public void addRGB(int dr, int dg, int db) {

		for (int i = 0;i < _height;i++) {
			for (int j = 0;j < _width;j++) {
				addRGB(j, i, dr, dg, db);
			}
		}

	}

	public void addHSB(int dh, int ds, int db) {

		for (int i = 0;i < _height;i++) {
			for (int j = 0;j < _width;j++) {
				addHSB(j, i, dh, ds, db);
			}
		}

	}

	public void post(int rd, int gd, int bd) {

		int rb[] = new int[256];
		int gb[] = new int[256];
		int bb[] = new int[256];

		double d, n;

		// 赤階調のテーブル作成
		double rn = (256.0 / (double)rd);
		double dr = 255.0 / (double)((double)rd - 1.0);

		for (int i = 0;i < rd;i++) {
			for (int j = (int)Math.floor(i * rn);j < (int)Math.floor((i + 1) * rn);j++) {
				rb[j] = (int)(dr * (double)i);
			}
		}

		// 緑階調のテーブル作成
		double gn = (256.0 / (double)gd);
		double dg = 255.0 / (double)((double)gd - 1.0);

		for (int i = 0;i < gd;i++) {
			for (int j = (int)Math.floor(i * gn);j < (int)Math.floor((i + 1) * gn);j++) {
				gb[j] = (int)(dg * (double)i);
			}
		}

		// 青階調のテーブル作成
		double bn = (256.0 / (double)bd);
		double db = 255.0 / (double)((double)bd - 1.0);

		for (int i = 0;i < bd;i++) {
			for (int j = (int)Math.floor(i * bn);j < (int)Math.floor((i + 1) * bn);j++) {
				bb[j] = (int)(db * (double)i);
			}
		}

		// 各ピクセルのRGB成分を変換
		for (int i = 0;i < _height;i++) {
			for (int j = 0;j < _width;j++) {

				setR(j, i, (int)rb[getR(j, i)]);
				setG(j, i, (int)gb[getG(j, i)]);
				setB(j, i, (int)bb[getB(j, i)]);

			}
		}

	}

	public void rgbFunc(double[] args) {

		if (args == null || args.length < 12) {
			return;
		}

		// 各ピクセルのRGB成分を変換
		for (int i = 0;i < _height;i++) {
			for (int j = 0;j < _width;j++) {

				double r = getR(j, i) * args[0] + getG(j, i) * args[1] + getB(j, i) * args[2] + args[3];
				double g = getR(j, i) * args[4] + getG(j, i) * args[5] + getB(j, i) * args[6] + args[7];
				double b = getR(j, i) * args[8] + getG(j, i) * args[9] + getB(j, i) * args[10] + args[11];

				setR(j, i, (int)r);
				setG(j, i, (int)g);
				setB(j, i, (int)b);

			}
		}

	}

	// 白黒化
	public void gray() {

		// 各ピクセルを白黒化
		for (int i = 0;i < _height;i++) {
			for (int j = 0;j < _width;j++) {

				// ピクセルの明度を算出
				int y = (int)(0.299 * getR(j, i) + 0.587 * getG(j, i) + 0.114 * getB(j, i));

				// RGB各成分を明度で統一
				setR(j, i, y);
				setG(j, i, y);
				setB(j, i, y);

			}
		}

	}

	public void bluer(int p0, int p1, int p2) {

		int[] pixels = new int[2 + _width * _height];

		if (pixels == null) {
			return;
		}

		pixels[0] = _width;
		pixels[1] = _height;

		for (int i = 0;i < _height;i++) {
			for (int j = 0;j < _width;j++) {
				pixels[2 + j + i * _width] = bluer(j, i, p0, p1, p2);
			}
		}

		assign(pixels);

	}

	public int bluer(int x, int y, int p0, int p1, int p2) {

		double pp0 = (double)p0 / 100.0;
		double pp1 = (double)p1 / 100.0;
		double pp2 = (double)p2 / 100.0;

		double r0 = 0, r1 = 0, r2 = 0;
		double g0 = 0, g1 = 0, g2 = 0;
		double b0 = 0, b1 = 0, b2 = 0;

		int rn1 = 0, rn2 = 0;
		int gn1 = 0, gn2 = 0;
		int bn1 = 0, bn2 = 0;

		for (int i = 0;i < 5;i++) {
			for (int j = 0;j < 5;j++) {

				int px = x + (j - 2);
				int py = y + (i - 2);

				if (px >= 0 && px < _width && py >= 0 && py < _height) {

					// 中心ピクセル
					if (i == 2 && j == 2) {

						r0 += getR(px, py) * pp0;

						g0 += getG(px, py) * pp0;

						b0 += getB(px, py) * pp0;

					}

					// 距離1のピクセル
					if ((i == 1 || i == 3) && (j == 1 || j == 3)) {

						r1 += getR(px, py) * pp1;
						rn1++;

						g1 += getG(px, py) * pp1;
						gn1++;

						b1 += getB(px, py) * pp1;
						bn1++;

					}

					// 距離2のピクセル
					if ((i == 0 || i == 4) && (j == 0 || j == 4)) {

						r2 += getR(px, py) * pp2;
						rn2++;

						g2 += getG(px, py) * pp2;
						gn2++;

						b2 += getB(px, py) * pp2;
						bn2++;

					}

				}

			}
		}

		int r = (int)(r0 + (r1 / rn1) + (r2 / rn2));
		int g = (int)(g0 + (g1 / gn1) + (g2 / gn2));
		int b = (int)(b0 + (b1 / bn1) + (b2 / bn2));

		if (r < 0) {
			r = 0;
		}

		if (r > 255) {
			r = 255;
		}

		if (g < 0) {
			g = 0;
		}

		if (g > 255) {
			g = 255;
		}

		if (b < 0) {
			b = 0;
		}

		if (b > 255) {
			b = 255;
		}

		return (r << 16) | (g << 8) | b;

	}

	public void unsharp(int range, double val) {

		int r, g, b, r1, g1, b1, r2, g2, b2, dr, dg, db, v;

		int[] mask = new int[_width * _height];

		for (int i = 0;i < _height;i++) {
			for (int j = 0;j < _width;j++) {

				r = g = b = v = 0;

				for (int k = -range; k <= range;k++) {
					for (int l = -range;l <= range;l++) {
						if (j + l >= 0 && j + l < _width && i + k >= 0 && i + k < _height) {

							r += getR(j + l, i + k);
							g += getG(j + l, i + k);
							b += getB(j + l, i + k);

							v++;

						}
					}
				}

				r /= v;
				g /= v;
				b /= v;

				mask[j + i * _width] = (r << 16) + (g << 8) + b;

			}

		}

		for (int i = 0;i < _height;i++) {
			for (int j = 0;j < _width;j++) {

				r1 = getR(j, i);
				g1 = getG(j, i);
				b1 = getB(j, i);

				r2 = (mask[j + i * _width] & 0x00ff0000) >> 16;
				g2 = (mask[j + i * _width] & 0x0000ff00) >> 8;
				b2 = mask[j + i * _width] & 0x000000ff;

				dr = r1 - r2;
				dg = g1 - g2;
				db = b1 - b2;

				setR(j, i, (int)(r1 + dr * val));
				setG(j, i, (int)(g1 + dg * val));
				setB(j, i, (int)(b1 + db * val));

			}
		}

	}

	public int[] getSubPixels(int x, int y, int width, int height) {

		if (x < 0 || y < 0 || width < 0 || height < 0 || x + width > _width || y + height > _height) {
			return null;
		}

		int[] pixels = new int[2 + width * height];

		pixels[0] = width;
		pixels[1] = height;

		if (pixels == null) {
			return null;
		}

		for (int i = 0;i < height;i++) {
			for (int j = 0;j < width;j++) {
				pixels[2 + x + j + (y + i) * width] = getPixel(x + j, y + i);
			}
		}

		return pixels;

	}

	public int[] getScaledPixels(int width, int height) {

		if (width < 1 || height < 1) {
			return null;
		}

		Image img = getImage();

		Image scaled = img.getScaledInstance(width, height, Image.SCALE_SMOOTH);

		return j8phImageCodec.getPixels(scaled);

	}

	public int[] getTiledPixels(int width, int height) {

		if (width < 1 || height < 1 || width > _width || height > _height) {
			return null;
		}

		int[] pixels = new int[2 + _width * _height];

		if (pixels == null) {
			return null;
		}

		int swidth = _width / width;
		int sheight = _height / height;

		int[] sub = getScaledPixels(swidth, sheight);

		if (sub == null) {
			return null;
		}

		pixels[0] = _width;
		pixels[1] = _height;

		for (int i = 0;i < _height;i++) {
			for (int j = 0;j < _width;j++) {

				int px = j / width;
				int py = i / height;

				if (px < swidth && py < sheight) {
					pixels[2 + j + i * _width] = sub[2 + px + py * swidth];
				} else {
					pixels[2 + j + i * _width] = getPixel(j, i);
				}

			}
		}

		return pixels;

	}

	public void gamma(double gm) {

		double r, g, b;

		for (int i = 0;i < _height;i++) {
			for (int j = 0;j < _width;j++) {

				r = getR(j, i);
				g = getG(j, i);
				b = getB(j, i);

				r = 255 * Math.pow((r / 255.0), (1.0 / gm));
				g = 255 * Math.pow((g / 255.0), (1.0 / gm));
				b = 255 * Math.pow((b / 255.0), (1.0 / gm));

				setR(j, i, (int)r);
				setG(j, i, (int)g);
				setB(j, i, (int)b);

			}
		}

	}

	public void oilPaint() {

		if (_width < 16 || _height < 16) {
			return;
		}

		post(12, 12, 12);

		int[] tiled1 = getTiledPixels(8, 8);

		if (tiled1 == null) {
			return;
		}

		int[] tiled2 = getTiledPixels(3, 3);

		if (tiled2 == null) {
			return;
		}

		post(8, 8, 8);

		for (int i = 0;i < _height;i++) {
			for (int j = 0;j < _width;j++) {

				int r = (getR(j, i) + ((tiled1[2 + j + i * _width] >> 16) & 0xff) + ((tiled2[2 + j + i * _width] >> 16) & 0xff)) / 3;
				int g = (getG(j, i) + ((tiled1[2 + j + i * _width] >> 8) & 0xff) + ((tiled2[2 + j + i * _width] >> 8) & 0xff)) / 3;
				int b = (getB(j, i) + (tiled1[2 + j + i * _width] & 0xff) + (tiled2[2 + j + i * _width] & 0xff)) / 3;

				setR(j, i, r);
				setG(j, i, g);
				setB(j, i, b);

			}
		}

		tiled1 = null;
		tiled2 = null;

		bluer(40, 30, 30);

		System.gc();

	}

	public void pcamera() {

		gray();

		if (_width < 16 || _height < 16) {
			return;
		}

		int[] tiled1 = getTiledPixels(7, 7);

		if (tiled1 == null) {
			return;
		}

		int[] tiled2 = getTiledPixels(3, 3);

		if (tiled2 == null) {
			return;
		}

		post(24, 24, 24);

		for (int i = 0;i < _height;i++) {
			for (int j = 0;j < _width;j++) {

				int r = (getR(j, i) * 2 + ((tiled1[2 + j + i * _width] >> 16) & 0xff) + ((tiled2[2 + j + i * _width] >> 16) & 0xff)) / 4;
				int g = (getG(j, i) * 2+ ((tiled1[2 + j + i * _width] >> 8) & 0xff) + ((tiled2[2 + j + i * _width] >> 8) & 0xff)) / 4;
				int b = (getB(j, i) * 2+ (tiled1[2 + j + i * _width] & 0xff) + (tiled2[2 + j + i * _width] & 0xff)) / 4;

				setR(j, i, r);
				setG(j, i, g);
				setB(j, i, b);

			}
		}

		tiled1 = null;
		tiled2 = null;

		unsharp(1, -0.5);
		bluer(20, 40, 40);

		System.gc();

	}

	public Image getImage() {

		BufferedImage img = new BufferedImage(_width, _height, BufferedImage.TYPE_INT_BGR);

		if (img == null) {
			return null;
		}

		for (int i = 0;i < _height;i++) {
			for (int j = 0;j < _width;j++) {
				img.setRGB(j, i, getPixel(j, i));
			}
		}

		return img;

	}

}
