import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;

public class PostDialog extends JDialog implements ActionListener, ChangeListener, ComponentListener {

	private j8ph _owner;

	private Box _views, _Rfields, _Gfields, _Bfields, _buttons;

	private PixelsViewer _viewO, _viewP;
	private JLabel _viewLabel;

	private JLabel _Rlabel, _Glabel, _Blabel;
	private JSlider _Rslider, _Gslider, _Bslider;
	private JCheckBox _chk;

	private JButton _btnExec, _btnCancel;

	private int _r, _g, _b;

	private int[] _original;
	private JIntFrameBuffer _preview;

	public PostDialog(j8ph owner, JIntFrameBuffer buf) {

		super(owner, "階調変更", true);

		_owner = owner;

		int w = buf.getWidth();
		int h = buf.getHeight();

		int width = 0;
		int height = 0;

		if (w > h) {

			if (w < j8ph.PREVIEW_SIZE) {
				width = w;
			} else {
				width = j8ph.PREVIEW_SIZE;
			}

			height = (h * width) / w;

		} else {

			if (h < j8ph.PREVIEW_SIZE) {
				height = h;
			} else {
				height = j8ph.PREVIEW_SIZE;
			}

			width = (w * height) / h;

		}

		_views = Box.createHorizontalBox();

		_original = buf.getScaledPixels(width, height);
		_preview = new JIntFrameBuffer(_original);

		_viewO = new PixelsViewer(_original);
		_viewLabel = new JLabel("→");
		_viewP = new PixelsViewer(_original);

		_views.add(_viewO);
		_views.add(_viewLabel);
		_views.add(_viewP);

		_Rfields = Box.createHorizontalBox();

		_Rlabel = new JLabel("R:256階調");
		_Rslider = new JSlider(2, 256, 256);
		_Rslider.addChangeListener(this);

		_Rfields.add(Box.createHorizontalStrut(4));
		_Rfields.add(_Rlabel);
		_Rfields.add(Box.createHorizontalStrut(4));
		_Rfields.add(_Rslider);

		_Gfields = Box.createHorizontalBox();

		_Glabel = new JLabel("G:256階調");
		_Gslider = new JSlider(2, 256, 256);
		_Gslider.addChangeListener(this);

		_Gfields.add(Box.createHorizontalStrut(4));
		_Gfields.add(_Glabel);
		_Gfields.add(Box.createHorizontalStrut(4));
		_Gfields.add(_Gslider);

		_Bfields = Box.createHorizontalBox();

		_Blabel = new JLabel("B:256階調");
		_Bslider = new JSlider(2, 256, 256);
		_Bslider.addChangeListener(this);

		_Bfields.add(Box.createHorizontalStrut(4));
		_Bfields.add(_Blabel);
		_Bfields.add(Box.createHorizontalStrut(4));
		_Bfields.add(_Bslider);

		_chk = new JCheckBox("RGB連動", true);

		_buttons = Box.createHorizontalBox();

		_btnExec = new JButton("実行");
		_btnExec.addActionListener(this);

		_btnCancel = new JButton("中止");
		_btnCancel.addActionListener(this);

		// ボタン欄作成
		_buttons.add(_btnExec);
		_buttons.add(Box.createHorizontalStrut(4));
		_buttons.add(_btnCancel);

		Container c = getContentPane();

		c.setLayout(new BoxLayout(c, BoxLayout.Y_AXIS));

		c.add(Box.createVerticalStrut(4));
		c.add(_views);
		c.add(Box.createVerticalStrut(4));
		c.add(_Rfields);
		c.add(Box.createVerticalStrut(4));
		c.add(_Gfields);
		c.add(Box.createVerticalStrut(4));
		c.add(_Bfields);
		c.add(Box.createVerticalStrut(4));
		c.add(_chk);
		c.add(Box.createVerticalStrut(8));
		c.add(_buttons);
		c.add(Box.createVerticalStrut(4));

		setResizable(false);
		pack();

		// アプリケーションの位置取得
		Rectangle rec = owner.getBounds(null);

		// アプリケーション中央座標算出
		int appCenterX = rec.x + rec.width / 2;
		int appCenterY = rec.y + rec.height / 2;

		// ダイアログをアプリケーション中央座標にあわせて配置
		setLocation(appCenterX - getWidth() / 2, appCenterY - getHeight() / 2);

		_r = -1;
		_g = -1;
		_b = -1;

		addComponentListener(this);

	}

	public int getR() {
		return _r;
	}

	public int getG() {
		return _g;
	}

	public int getB() {
		return _b;
	}

	public void actionPerformed(ActionEvent e) {

		if (e.getSource() == _btnExec) {

			_r = _Rslider.getValue();
			_g = _Gslider.getValue();
			_b = _Bslider.getValue();

			hide();

		}

		if (e.getSource() == _btnCancel) {

			_r = -1;
			_g = -1;
			_b = -1;

			hide();

		}

	}

	public void stateChanged(ChangeEvent e) {

		int r = _Rslider.getValue();
		int g = _Gslider.getValue();
		int b = _Bslider.getValue();

		_preview.assign(_original);

		_preview.post(r, g, b);

		_viewP.setPixels(_preview.getPixels());

		repaint();

		if (e.getSource() == _Rslider) {

			int val = _Rslider.getValue();

			_Rlabel.setText("R:" + String.valueOf(val) + "階調");

			if (_chk.isSelected()) {

				_Gslider.setValue(val);
				_Glabel.setText("G:" + String.valueOf(val) + "階調");

				_Bslider.setValue(val);
				_Blabel.setText("B:" + String.valueOf(val) + "階調");

			}

		}

		if (e.getSource() == _Gslider) {

			int val = _Gslider.getValue();

			_Glabel.setText("G:" + String.valueOf(val) + "階調");

			if (_chk.isSelected()) {

				_Rslider.setValue(val);
				_Rlabel.setText("R:" + String.valueOf(val) + "階調");

				_Bslider.setValue(val);
				_Blabel.setText("B:" + String.valueOf(val) + "階調");

			}

		}

		if (e.getSource() == _Bslider) {

			int val = _Bslider.getValue();

			_Blabel.setText("B:" + String.valueOf(val) + "階調");

			if (_chk.isSelected()) {

				_Rslider.setValue(val);
				_Rlabel.setText("R:" + String.valueOf(val) + "階調");

				_Bslider.setValue(val);
				_Blabel.setText("B:" + String.valueOf(val) + "階調");

			}

		}

	}

	public void componentHidden(ComponentEvent e) {
	}

	public void componentMoved(ComponentEvent e) {
		_owner.repaint();
	}

	public void componentResized(ComponentEvent e) {
	}

	public void componentShown(ComponentEvent e) {
	}

}

