import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;

public class RGBFuncDialog extends JDialog implements ActionListener, ComponentListener {

	private j8ph _owner;

	private PixelsViewer _viewO, _viewP;

	private JTextField[][] _rgbFields;

	private JButton _btnExec, _btnPreview, _btnCancel;

	private Box _views, _Rfields, _Gfields, _Bfields, _buttons;

	private int[] _original;
	private JIntFrameBuffer _preview;

	private double[] _rgbVal;

	public RGBFuncDialog(j8ph owner, JIntFrameBuffer buf) {

		super(owner, "RGB変換", true);

		_owner = owner;

		int w = buf.getWidth();
		int h = buf.getHeight();

		int width = 0;
		int height = 0;

		if (w > h) {

			if (w < j8ph.PREVIEW_SIZE) {
				width = w;
			} else {
				width = j8ph.PREVIEW_SIZE;
			}

			height = (h * width) / w;

		} else {

			if (h < j8ph.PREVIEW_SIZE) {
				height = h;
			} else {
				height = j8ph.PREVIEW_SIZE;
			}

			width = (w * height) / h;

		}

		_views = Box.createHorizontalBox();

		_original = buf.getScaledPixels(width, height);
		_preview = new JIntFrameBuffer(_original);

		_viewO = new PixelsViewer(_original);
		_viewP = new PixelsViewer(_original);

		_views.add(_viewO);
		_views.add(new JLabel("→"));
		_views.add(_viewP);

		_rgbFields = new JTextField[3][4];

		for (int i = 0;i < 3;i++) {
			for (int j = 0;j < 4;j++) {

				_rgbFields[i][j] = new JTextField();

				if (i == j) {
					_rgbFields[i][j].setText("1.0");
				} else {
					_rgbFields[i][j].setText("0.0");
				}

			}
		}

		_Rfields = Box.createHorizontalBox();

		_Rfields.add(new JLabel("R="));
		_Rfields.add(_rgbFields[0][0]);
		_Rfields.add(new JLabel("R + "));
		_Rfields.add(_rgbFields[0][1]);
		_Rfields.add(new JLabel("G + "));
		_Rfields.add(_rgbFields[0][2]);
		_Rfields.add(new JLabel("B + "));
		_Rfields.add(_rgbFields[0][3]);

		_Gfields = Box.createHorizontalBox();

		_Gfields.add(new JLabel("R="));
		_Gfields.add(_rgbFields[1][0]);
		_Gfields.add(new JLabel("R + "));
		_Gfields.add(_rgbFields[1][1]);
		_Gfields.add(new JLabel("G + "));
		_Gfields.add(_rgbFields[1][2]);
		_Gfields.add(new JLabel("B + "));
		_Gfields.add(_rgbFields[1][3]);

		_Bfields = Box.createHorizontalBox();

		_Bfields.add(new JLabel("R="));
		_Bfields.add(_rgbFields[2][0]);
		_Bfields.add(new JLabel("R + "));
		_Bfields.add(_rgbFields[2][1]);
		_Bfields.add(new JLabel("G + "));
		_Bfields.add(_rgbFields[2][2]);
		_Bfields.add(new JLabel("B + "));
		_Bfields.add(_rgbFields[2][3]);

		_buttons = Box.createHorizontalBox();

		_btnExec = new JButton("実行");
		_btnExec.addActionListener(this);

		_btnPreview = new JButton("確認");
		_btnPreview.addActionListener(this);

		_btnCancel = new JButton("中止");
		_btnCancel.addActionListener(this);

		// ボタン欄作成
		_buttons.add(_btnExec);
		_buttons.add(Box.createHorizontalStrut(4));
		_buttons.add(_btnPreview);
		_buttons.add(Box.createHorizontalStrut(4));
		_buttons.add(_btnCancel);

		Container c = getContentPane();

		c.setLayout(new BoxLayout(c, BoxLayout.Y_AXIS));

		c.add(Box.createVerticalStrut(4));
		c.add(_views);
		c.add(Box.createVerticalStrut(4));
		c.add(_Rfields);
		c.add(Box.createVerticalStrut(4));
		c.add(_Gfields);
		c.add(Box.createVerticalStrut(4));
		c.add(_Bfields);
		c.add(Box.createVerticalStrut(4));
		c.add(_buttons);
		c.add(Box.createVerticalStrut(4));

		setResizable(false);
		pack();

		// アプリケーションの位置取得
		Rectangle rec = owner.getBounds(null);

		// アプリケーション中央座標算出
		int appCenterX = rec.x + rec.width / 2;
		int appCenterY = rec.y + rec.height / 2;

		// ダイアログをアプリケーション中央座標にあわせて配置
		setLocation(appCenterX - getWidth() / 2, appCenterY - getHeight() / 2);

		addComponentListener(this);

	}

	public double[] getValue() {

		return _rgbVal;

	}

	public void actionPerformed(ActionEvent e) {

		if (e.getSource() == _btnExec) {

			_rgbVal = new double[12];

			for (int i = 0;i < 3;i++) {
				for (int j = 0;j < 4;j++) {

					try {

						double val = Double.parseDouble(_rgbFields[i][j].getText());

						_rgbVal[j + i * 4] = val;

					} catch (Exception ex) {

						_rgbVal = null;

						return;

					}

				}
			}

			hide();

		}

		// 「プレビュー」ボタン
		if (e.getSource() == _btnPreview) {

			double[] rgbVal = new double[12];

			for (int i = 0;i < 3;i++) {
				for (int j = 0;j < 4;j++) {

					try {

						double val = Double.parseDouble(_rgbFields[i][j].getText());

						rgbVal[j + i * 4] = val;

					} catch (Exception ex) {

						rgbVal = null;

						return;

					}

				}
			}

			_preview.assign(_original);

			_preview.rgbFunc(rgbVal);

			_viewP.setPixels(_preview.getPixels());

			repaint();

		}

		if (e.getSource() == _btnCancel) {

			_rgbVal = null;

			hide();

		}

	}

	public void componentHidden(ComponentEvent e) {
	}

	public void componentMoved(ComponentEvent e) {
		_owner.repaint();
	}

	public void componentResized(ComponentEvent e) {
	}

	public void componentShown(ComponentEvent e) {
	}

}
